package org.greenstone.gsdl3.collection;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.core.*;
import org.greenstone.gsdl3.service.*;


// java XML classes we're using
import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList; 

import java.io.File;
import java.util.HashMap;

/**
 * Represents a collection in Greenstone.
 *
 * Implements ModuleInterface. Contains a list of services supported by the collection, as well as collection metadata
 *
 * @see ModuleInterface
 */
public class Collection 
    implements ModuleInterface {

    /** base directory for the site */
    protected String site_home_ = null;

    /** The name of the collection - this must be the name of the collection 
     * directory inside collect */
    protected String collection_name_ = null;

    /** The map of services.
     *
     * Maps Services to ServiceModule objects
     * @see ServiceModule
     *
     */
    protected HashMap service_map_=null;

    /** XML converter for String to DOM and vice versa */
    protected XMLConverter converter_=null;

    /** container doc for description elements */
    protected Document doc_ = null;
    /** list of services */
    protected Element service_info_ = null;
    /** list of Metadata */
    protected Element meta_info_ = null;
    /** full description */
    protected Element description_ = null;

    public void setSiteHome(String home) {
	site_home_ = home;
    }

    public void setCollectionName(String name) {
	collection_name_ = name;
	description_.setAttribute("name", name);
    }

    public Collection() {
	service_map_ = new HashMap();
	converter_ = new XMLConverter();
	doc_ = converter_.newDOM();
	description_ = doc_.createElement("collection");
	service_info_ = doc_.createElement("serviceList");
	meta_info_ = doc_.createElement("metadataList");
	
	
    }
	
    /**
     * Configures the collection.
     *
     * gsdlHome and collectionName must be set before configure is called.
     *
     * the file buildcfg.xml is located in gsdlHome/collect/collectionName
     * collection metadata is obtained, and services loaded.
     *
     * @return true/false on success/fail
     */
    public boolean configure() {
	
	if (site_home_ == null || collection_name_== null) {
	    System.err.println("site_home_ and collection_name_ must be set before configure called!");
	    return false;
	}
	// read the collection build configuration file
	File config_file_ = GSFile.collectionBuildConfigFile(site_home_, collection_name_);

	if (!config_file_.exists()) {
	    System.err.println(config_file_+" does not exist");
	    System.err.println("couldn't configure collection: "+collection_name_);
	    return false;
	}

	try {
	    
	    Document doc = converter_.getDOM(config_file_);
	    
	    // create all the services
	    NodeList nodes = doc.getElementsByTagName("serviceModule");

	    // the xml request to send to each serviceModule to query what
	    // services it provides
	    Element request = doc_.createElement("request");
	    request.setAttribute("type", "describe");
	    request.setAttribute("info", "serviceList");

	    if (nodes.getLength()==0) {
		System.err.println("Collection configuration error: collection "+collection_name_+" has no services!");
		return false;
	    }

	    for(int i=0; i<nodes.getLength(); i++) {
		Element n = (Element)nodes.item(i);
		String servicetype = n.getAttribute("name");
		
		ServiceModule s = (ServiceModule)Class.forName("org.greenstone.gsdl3.service."+servicetype).newInstance();
		s.setSiteHome(site_home_);
		s.setCollectionName(collection_name_);
		// pass the xml node to the service for configuration
		s.configure(n);

		// find out the supported service types for this service module
		Node types = s.process(request);
		NodeList typenodes = ((Element)types).getElementsByTagName("service");	   

		for (int j=0; j<typenodes.getLength();j++) {
		    String service = ((Element) typenodes.item(j)).getAttribute("name");	    
		    service_map_.put(service, s);

		    // also add info to the ServiceInfo XML element
		    service_info_.appendChild(doc_.importNode(typenodes.item(j), true));
		}
		
	    }

	    // get the metadata - for now just add it to the list
	    NodeList metanodes = doc.getElementsByTagName("metadata");

	    if (metanodes.getLength()>0) { 	
		for(int k=0; k<metanodes.getLength(); k++) {
		    meta_info_.appendChild(doc_.importNode(metanodes.item(k),true));
		}
	    }	     

	} catch (Exception e) {
	    System.out.println("Collection configure exception: "+e.getMessage() + e.getClass() );
	    e.printStackTrace();
	    return false;
	}

	// add elements to description
	description_.appendChild(service_info_);
	description_.appendChild(meta_info_);
	return true;
    }


    /**
     * Process an XML document - uses Strings 
     *  just calls process(Node).
     *
     * @param in the Document to process - a string
     * @return the resultant document as a string - contains any error messages
     * @see String
     */
    public String process(String in) {

	Document doc = converter_.getDOM(in);
	Element e = doc.getDocumentElement();
	
	Node res = process(e);
	return converter_.getString(res);
	
    }
    
    /** process XML as Node
     *
     */
    public Node process(Node xml_in) {

	// xml_in should always be an Element - should check this
	Element xml = (Element)xml_in;
	String to = xml.getAttribute("to");

	// the collection name should be first, check, then remove
	String col = GSPath.getFirstLink(to);
	if (col.equals(collection_name_)){
	    to = GSPath.removeFirstLink(to);
	    xml.setAttribute("to", to);
	} else {
	    System.out.println("Collection error: collection name wrong! was "+col+" should have been "+collection_name_);
	    return null;
	}

	if (to.equals("")) { // this command is for me
	    String type = xml.getAttribute("type");
	    if (type.equals("describe")) {
		// create a response with the description inside it
		Element response = doc_.createElement("response");
		response.setAttribute("from", collection_name_);
		response.setAttribute("type", "describe");
		response.appendChild(description_);
		return response;
	    } else {
		System.out.println("Collection: error, cant handle request:");
		System.out.println(converter_.getString(xml));
		return null;
	    }
	    
	} else { // the request is for one of my services
	    String service = GSPath.getFirstLink(to);
	    
	    if (service_map_.containsKey(service)) {
		return ((ServiceModule)service_map_.get(service)).process(xml);
	    } else {
		System.err.println("Collection "+collection_name_+": non-existant service in:\n"+converter_.getString(xml));
		return null;
	    }

	}
    }
	 
}
